@@ -6,21 +6,28 @@ from json import dumps
 from json import load
 from os.path import getsize
 from pathlib import Path
+from re import Pattern
+from re import compile as re_compile
+from re import match
 from re import sub
 from shutil import get_terminal_size
 from sys import stderr
 from sys import stdout
+from textwrap import wrap
 from typing import Any
 from typing import Callable
 from typing import Iterable
 from typing import TextIO
 
+from bs4 import BeautifulSoup
 from click import Argument
 from click import BadParameter
+from click import Choice
 from click import Context
 from click import File
 from click import IntRange
 from click import Option
+from click import Parameter
 from click import Path as PathClick
 from click import argument
 from click import confirmation_option
@@ -56,9 +63,11 @@ from .util import color_option
 from .util import database_callback
 from .util import database_exists_option
 from .util import docstring_format
+from .util import get_param
 from .util import help_option
 from .. import __name__ as __prog_name__
 from ..__version__ import __version__
+from ..downloader import sort_set
 
 
 class Output(str, Enum):
@@ -85,8 +94,8 @@ class TableChoice(CompleteChoice):
 
 class SearchOrderChoice(CompleteChoice):
     completion_items: list[CompletionItem] = [
-        CompletionItem("asc", "Ascending order"),
-        CompletionItem("desc", "Descending order"),
+        CompletionItem("asc", help="Ascending order"),
+        CompletionItem("desc", help="Descending order"),
     ]
 
 
@@ -100,26 +109,77 @@ class SearchOutputChoice(CompleteChoice):
     ]
 
 
-class ExportOutputChoice(CompleteChoice):
-    completion_items: list[CompletionItem] = [
-        CompletionItem(Output.csv.value, help="CSV format (comma separated)"),
-        CompletionItem(Output.tsv.value, help="TSV format (tab separated)"),
-        CompletionItem(Output.json.value, help="JSON format"),
+class ColumnsChoice(Choice):
+    completion_items: list[tuple[str, CompletionItem]] = []
+
+    def __init__(self):
+        super().__init__(sort_set([i.value for _, i in self.completion_items]), False)
+
+    def shell_complete(self, ctx: Context, param: Parameter, incomplete: str) -> list[CompletionItem]:
+        table: str = ctx.params.get("table", None) or (ctx.args or [""])[0]
+        return [i for t, i in self.completion_items
+                if (not table or t.lower() == table.lower()) and i.value.lower().startswith(incomplete.lower())]
+
+
+class SortColumnsChoice(ColumnsChoice):
+    completion_items: list[tuple[str, CompletionItem]] = [
+        *[(submissions_table, CompletionItem(c.name, help=f"{submissions_table}:{c.name}"))
+          for c in SubmissionsColumns.as_list()],
+        *[(journals_table, CompletionItem(c.name, help=f"{journals_table}:{c.name}"))
+          for c in JournalsColumns.as_list()],
+        *[(users_table, CompletionItem(c.name, help=f"{users_table}:{c.name}"))
+          for c in UsersColumns.as_list()],
+    ]
+
+
+class SearchColumnsChoice(ColumnsChoice):
+    completion_items: list[tuple[str, CompletionItem]] = [
+        *SortColumnsChoice.completion_items,
+        (submissions_table, CompletionItem("@", help=f"{submissions_table}:ALL")),
+        (journals_table, CompletionItem("@", help=f"{journals_table}:ALL")),
+        (users_table, CompletionItem("@", help=f"{users_table}:ALL")),
     ]
 
 
 def serializer(obj: object) -> object:
-    if isinstance(obj, Iterable):
-        return list(obj)
-    else:
-        return str(obj)
+    return list(obj) if isinstance(obj, Iterable) else str(obj)
 
 
-def column_callback(_ctx: Context, _param: Option, value: tuple[str]) -> list[tuple[str, int]]:
-    return [((vs := v.split(",", 1))[0], int(vs[1])) if "," in v else (v, 0) for v in map(str.strip, value) if v]
+def date_callback(ctx: Context, param: Parameter, value: str) -> tuple[str | None, ...] | None:
+    if not value:
+        return None
+    elif m := match(r"(?:(\d{4})(?:-(\d\d?)(?:-(\d\d?))?)?)?[ ]?(?:(\d\d?)(?::(\d\d?)(?::(\d\d?))?)?)?", value):
+        return m.groups()
+    raise BadParameter("Not a valid date format.", ctx, param)
 
 
-def id_callback(ctx: Context, param: Argument, value: tuple[str, ...]) -> tuple[str | int, ...]:
+def column_callback(ctx: Context, param: Option, value: tuple[str]) -> tuple[str]:
+    table_columns: list[str] = []
+    if (table := ctx.params["table"]) == submissions_table:
+        table_columns = [c.name for c in SubmissionsColumns.as_list()]
+    elif table == journals_table:
+        table_columns = [c.name for c in JournalsColumns.as_list()]
+    elif table == users_table:
+        table_columns = [c.name for c in UsersColumns.as_list()]
+    if (col := next((c for c in value if c not in table_columns and c != "@"), None)) is not None:
+        raise BadParameter(f"{col!r} is not one of {', '.join(map(repr, table_columns))}.", ctx, param)
+    return value
+
+
+def sort_callback(ctx: Context, param: Option, value: tuple[tuple[str, str]]) -> tuple[tuple[str, str]]:
+    column_callback(ctx, param, tuple(c for c, _ in value))
+    return value
+
+
+def table_width_callback(ctx: Context, param: Option, value: str | None) -> tuple[int]:
+    values: list[str] = list(map(str.strip, value.split(","))) if value is not None else []
+    if (err := next(filter(lambda v: not v.isdigit(), values), None)) is not None:
+        raise BadParameter(f"{err!r} is not a valid integer.", ctx, param)
+    return tuple(map(int, values))
+
+
+def id_callback(ctx: Context, param: Argument, value: tuple[str, ...] | str) -> tuple[str | int, ...]:
+    value = value if isinstance(value, tuple) else (value,)
     if (t := ctx.params["table"]).lower() == users_table.lower():
         return value
     elif any(not v.isdigit() for v in value):
@@ -190,9 +250,9 @@ def print_csv(results: Cursor, file: TextIO, delimiter: str) -> int:
     results_total: int = 0
     writer = csv_writer(file, delimiter=delimiter)
     writer.writerow([c.name for c in results.columns])
-    for row in results.entries:
+    for row in results.tuples:
         results_total += 1
-        writer.writerow(map(format_value, row.values()))
+        writer.writerow(map(format_value, row))
     return results_total
 
 
@@ -212,7 +272,7 @@ def print_json(results: Cursor, file: TextIO) -> int:
 def search(table: Table, headers: list[str], query: str, sort: tuple[tuple[str, str]], limit: int | None,
            offset: int | None, sql: bool) -> tuple[Cursor, tuple[str, list[str]]]:
     cols_table: list[str] = [c.name for c in table.columns]
-    query_elems, values = ([query], []) if sql else query_to_sql(
+    query_elems, values = ([query], []) if sql or not query.strip() else query_to_sql(
         query, "any", [*map(str.lower, {*cols_table, "any"} - {"ID", "AUTHOR", "USERNAME"})],
         {"author": "replace(author, '_', '')", "any": f"({'||'.join(cols_table)})"})
     query = " ".join(query_elems)
@@ -220,6 +280,69 @@ def search(table: Table, headers: list[str], query: str, sort: tuple[tuple[str,
             (query, values))
 
 
+def html_to_ansi(html: str, *, root: bool = False) -> str:
+    html = sub("[\n\r]+", "", html) if root else html
+    html_parsed: BeautifulSoup = BeautifulSoup(html, "lxml")
+    width: int = get_terminal_size((0, 0)).columns
+    for img in html_parsed.select("img"):
+        img.replaceWith("")
+    for br in html_parsed.select("br"):
+        br.replaceWith("\n")
+    for hr in html_parsed.select("hr"):
+        hr.replaceWith(f"\n{'-' * (width or 40):^{(width or 1) - 1}}\n")
+    for [tag_name, tag_style] in (("i", italic), ("strong", bold), ("u", underline), ("s", strikethrough)):
+        for tag in html_parsed.select(tag_name):
+            for child in tag.select("*"):
+                child.replaceWith(html_to_ansi(str(child)))
+            tag.replaceWith(f"{tag_style}{tag.text}{reset}")
+    for a in html_parsed.select("a"):
+        for child in a.select("*"):
+            child.replaceWith(html_to_ansi(str(child)))
+        a.replaceWith(f"[{a.text.strip()}]({a['href']})")
+    for span in html_parsed.select("span.bbcode[style*=color]"):
+        for child in span.select("*"):
+            child.replaceWith(html_to_ansi(str(child)))
+        if (color := m[1] if (m := match(r".*color: ([^ ;]+).*", span.attrs["style"])) else "") in colors_dict:
+            color = colors_dict[color]
+        elif color in css_colors or color.startswith("#"):
+            color = hex_to_ansi(css_colors.get(color, color), truecolor=supports_truecolor)
+        span.replaceWith(f"{color}{span.text}{reset if color else ''}")
+    for span in html_parsed.select("span.bbcode_quote_name"):
+        for child in span.select("*"):
+            child.replaceWith(html_to_ansi(str(child)))
+        span.replaceWith(f"{bold}{italic}{span.text}{reset}")
+    for span in html_parsed.select("span.bbcode_quote"):
+        for child in span.select("*"):
+            child.replaceWith(html_to_ansi(str(child)))
+        width_: int = (width or 80) - 1
+        span.replaceWith("\n".join(line2
+                                   for line in span.text.splitlines()
+                                   for line2 in wrap(line, width_, initial_indent=" ", subsequent_indent=" ") or [""]))
+    for code in html_parsed.select("code.bbcode_center,code.bbcode_right") if width else []:
+        for child in code.select("*"):
+            child.replaceWith(html_to_ansi(str(child)))
+        formatter: str = "^" if "bbcode_center" in code["class"] else ">"
+        code.replaceWith("\n".join(f"{line2:{formatter}{(width or 1) - 1}}"
+                                   for line in code.text.splitlines()
+                                   for line2 in wrap(line, width)))
+    for child in html_parsed.select("*"):
+        child.replaceWith(child.text)
+    return html_parsed.text
+
+
+def view_entry(entry: dict[str, Any], html_fields: list[str], *, raw_html: bool = False) -> str:
+    outputs: list[str] = []
+    padding: int = max(map(len, entry.keys()))
+    for field, value in ((k, v) for k, v in entry.items() if k not in html_fields):
+        output = f"{blue}{field:<{padding}}{reset}: " + \
+                 (("\n" + (" " * (padding + 2))).join(value) if isinstance(value, (list, set)) else str(value).strip())
+        outputs.append(output)
+    for field, value in zip(html_fields, map(entry.__getitem__, html_fields)):
+        outputs.append(f"{blue}{field:<{padding}}{reset}:\n" +
+                       (value if raw_html else html_to_ansi(value, root=True)).strip())
+    return "\n".join(outputs)
+
+
 @group("database", cls=CustomHelpColorsGroup, short_help="Operate on the database.", no_args_is_help=True)
 @color_option
 @help_option
@@ -288,18 +411,23 @@ def database_info(ctx: Context, database: Callable[..., Database]):
 
 
 @database_app.command("history", short_help="Show database history.")
-@option("--filter", "_filter", metavar="FILTER", type=str, default="", callback=lambda _c, _p, v: v.lower(),
+@option("--filter", "_filter", metavar="FILTER", type=str, default=None,
         help=f"Show entries containing {yellow}FILTER{reset}.")
+@option("--filter-date", metavar="DATE", type=str, default=None, callback=date_callback,
+        help=f"Filter by {yellow}DATE{reset}.")
 @option("--clear", is_flag=True, default=False, required=False, help="Clear entries.")
 @database_exists_option
 @color_option
 @help_option
 @pass_context
 @docstring_format()
-def database_history(ctx: Context, database: Callable[..., Database], clear: bool, _filter: str):
+def database_history(ctx: Context, database: Callable[..., Database], clear: bool, _filter: str | None,
+                     filter_date: tuple[str | None, ...] | None):
     """
     Show database history. History events can be filtered using the {yellow}--filter{reset} option to match events that
-    contain {yellow}FILTER{reset} (the match is performed case-insensitively).
+    contain {yellow}FILTER{reset} (the match is performed case-insensitively). The {yellow}--filter-date{reset} option
+    allows filtering by date. The {yellow}DATE{reset} value must be in the format {cyan}YYYY-MM-DD HH:MM:SS{reset}, any
+    component can be omitted for generalised results.
 
     Using the {yellow}--clear{reset} option will delete all history entries, or the ones containing
     {yellow}FILTER{reset} if the {yellow}--filter{reset} option is used.
@@ -307,13 +435,15 @@ def database_history(ctx: Context, database: Callable[..., Database], clear: boo
 
     db: Database = database()
 
-    history: Iterable[tuple[datetime, str]]
+    history: Iterable[tuple[datetime, str]] = db.history.select().tuples
     removed: int = 0
 
     if _filter:
-        history = ((t, e) for t, e in db.history.select().tuples if _filter in e.lower())
-    else:
-        history = db.history.select().tuples
+        history = ((t, e) for t, e in history if _filter.lower() in e.lower())
+    if filter_date:
+        filter_exp: Pattern = re_compile("{:>02}-{:>02}-{:>02} {:>02}:{:>02}:{:>02}"
+                                         .format(*map(lambda d: d or r"\d+", filter_date)))
+        history = ((t, e) for t, e in history if match(filter_exp, f"{t:%Y-%m-%d %H:%M:%S}"))
 
     try:
         for t, e in history:
@@ -332,15 +462,17 @@ def database_history(ctx: Context, database: Callable[..., Database], clear: boo
 @database_app.command("search", short_help="Search database entries.", no_args_is_help=True)
 @argument("table", nargs=1, required=True, is_eager=True, type=TableChoice())
 @argument("query", nargs=-1, required=False, callback=lambda _c, _p, v: " ".join(v))
-@option("--column", metavar="<COLUMN[,WIDTH]>", type=str, multiple=True, callback=column_callback,
+@option("--column", metavar="COLUMN", type=SearchColumnsChoice(), multiple=True, callback=column_callback,
         help=f"Select {yellow}COLUMN{reset} and use {yellow}WIDTH{reset} in table output.")
-@option("--sort", metavar="<COLUMN [asc|desc]>", multiple=True, type=(str, SearchOrderChoice()),
-        help=f"Sort by {yellow}COLUMN{reset}.")
+@option("--sort", metavar="<COLUMN [asc|desc]>", multiple=True, type=(SortColumnsChoice(), SearchOrderChoice()),
+        help=f"Sort by {yellow}COLUMN{reset}.", callback=sort_callback)
 @option("--limit", type=IntRange(0, min_open=True), help="Limit query results.")
 @option("--offset", type=IntRange(0, min_open=True), help="Offset query results.")
 @option("--sql", is_flag=True, help="Treat query as SQLite WHERE statement.")
 @option("--show-sql", is_flag=True, help="Show generated SQLite WHERE statement.")
 @option("--output", default="table", type=SearchOutputChoice(), help="Specify output type.")
+@option("--table-widths", metavar="WIDTH,[WIDTH...]", type=str, callback=table_width_callback,
+        help="Specify width of table columns.")
 @option("--ignore-width", is_flag=True, help="Ignore terminal width when printing results in table format.")
 @option("--total", is_flag=True, help="Print number of results.")
 @database_exists_option
@@ -349,20 +481,20 @@ def database_history(ctx: Context, database: Callable[..., Database], clear: boo
 @pass_context
 @docstring_format(c=cyan, i=italic, r=reset, prog_name=__prog_name__, version=__version__,
                   outputs="\n    ".join(f" * {s.value}\t{s.help}" for s in SearchOutputChoice.completion_items))
-def database_search(ctx: Context, database: Callable[..., Database], table: str, query: str,
-                    column: tuple[tuple[str, int]], sort: tuple[tuple[str, str]], limit: int | None, offset: int | None,
-                    sql: bool, show_sql: bool, output: str, ignore_width: bool, total: bool):
+def database_search(ctx: Context, database: Callable[..., Database], table: str, query: str, column: tuple[str],
+                    sort: tuple[tuple[str, str]], limit: int | None, offset: int | None, sql: bool, show_sql: bool,
+                    output: str, table_widths: tuple[int], ignore_width: bool, total: bool):
     """
     Search the database using queries, and output in different formats.
 
     The default output format is a table with only the most relevant columns displayed for each entry. To override the
     displayed column, or change their width, use the {yellow}--column{reset} option to select which columns will be
-    displayed (SQLite statements are supported). The optional {yellow}WIDTH{reset} value can be added to format that
-    specific column when the output is set to {cyan}table{reset}.
+    displayed (SQLite statements are supported). The optional {yellow}WIDTH{reset} values can be added to format the
+    width of the specified columns when the output is set to {cyan}table{reset} and the {yellow}--column{reset} option
+    is used.
 
     To output all columns and entries of a table, {yellow}COLUMN{reset} and {yellow}QUERY{reset} values can be set to
-    {cyan}@{reset} and {cyan}%{reset} respectively. However, the {yellow}database export{reset} command is better
-    suited for this task.
+    {cyan}@{reset} and {cyan}%{reset} respectively.
 
     For a list of columns, see {yellow}database{reset} help.
 
@@ -413,18 +545,21 @@ def database_search(ctx: Context, database: Callable[..., Database], table: str,
 
     db: Database = database()
     db_table: Table = get_table(db, table)
-    default_headers: list[tuple[str, int]] = []
+    headers: list[tuple[str, int]] = [*zip(map(str.upper, column), (*table_widths, *([0] * len(column))))]
 
     if table in (submissions_table, journals_table):
-        default_headers = [(SubmissionsColumns.ID.name, 10), (SubmissionsColumns.AUTHOR.name, 16),
-                           (SubmissionsColumns.DATE.name, 16), (SubmissionsColumns.TITLE.name, 0)]
+        headers = headers or [*zip(cols := [SubmissionsColumns.ID.name, SubmissionsColumns.AUTHOR.name,
+                                            SubmissionsColumns.DATE.name, SubmissionsColumns.TITLE.name],
+                                   ((*table_widths, *([0] * len(cols))) if table_widths else (10, 16, 16, 0)))]
         sort = sort or ((SubmissionsColumns.ID.name, "desc"),)
     elif table == users_table:
-        default_headers = [(UsersColumns.USERNAME.name, 40), (UsersColumns.FOLDERS.name, 0)]
+        headers = headers or [*zip(cols := [UsersColumns.USERNAME.name, UsersColumns.FOLDERS.name],
+                                   ((*table_widths, *([0] * len(cols))) if table_widths else (40, 0)))]
         sort = sort or ((UsersColumns.USERNAME.name, "ASC"),)
 
-    headers: list[tuple[str, int]] = [(c.upper(), w) for c, w in column] if column else default_headers
-    headers = [(c, 0) for c in db_table.columns] if any(h == "@" for h, _ in headers) else headers
+    if any(h == "@" for h, _ in headers):
+        headers = list(zip([c.name for c in db_table.columns], (*table_widths, *([0] * len(db_table.columns)))))
+
     headers[-1] = (headers[-1][0], 0)
     results, [query, values] = search(db_table, [h for h, _ in headers], query, sort, limit, offset, sql)
     results_total: int = 0
@@ -447,52 +582,39 @@ def database_search(ctx: Context, database: Callable[..., Database], table: str,
         echo(f"{blue}Total{reset}: {yellow}{results_total}{reset}", color=ctx.color)
 
 
-@database_app.command("export", short_help="Export all entries in a table.", no_args_is_help=True)
+@database_app.command("view", short_help="View and entry.", no_args_is_help=True)
 @argument("table", nargs=1, required=True, is_eager=True, type=TableChoice())
-@argument("output", nargs=1, required=True, type=ExportOutputChoice())
-@argument("file", nargs=1, required=False, default=stdout, type=File("w"))
-@option("--column", metavar="COLUMN", type=str, multiple=True, help=f"Select {yellow}COLUMN{reset}.")
-@option("--sort", metavar="<COLUMN [asc|desc]>", multiple=True, type=(str, SearchOrderChoice()),
-        help=f"Sort by {yellow}COLUMN{reset}.")
-@option("--total", is_flag=True, help="Print number of results.")
+@argument("id_", metavar="ID", nargs=1, required=True, type=str, callback=id_callback)
+@option("--raw-content", is_flag=True, default=False, help="Do not format HTMl fields.")
 @database_exists_option
 @color_option
 @help_option
 @pass_context
-@docstring_format(outputs="\n    ".join(f" * {s.value}\t{s.help}" for s in ExportOutputChoice.completion_items))
-def database_export(ctx: Context, database: Callable[..., Database], table: str, output: str, file: TextIO,
-                    column: tuple[str], sort: tuple[tuple[str, str]], total: bool):
+def database_view(ctx: Context, database: Callable[..., Database], table: str, id_: tuple[str | int, ...],
+                  raw_content: bool):
     """
-    Export all entries in a table to a file. The {yellow}FILE{reset} argument can be omitted to print the results
-    directly in the terminal. The results total is not printed to file if a file is used.
+    View a single entry in the terminal. Submission descriptions, journal contents, and user profile pages are rendered
+    and formatted.
 
-    By default, all columns of the table are selected, but this can be overridden with the {yellow}--column{reset}
-    option (SQLite statements are supported).
+    Formatting is limited to alignment, horizontal lines, quotes, links, color, and emphasis. To view the properly
+    formatted HTML content, use the {yellow}server{reset} command. Formatting can be disabled with the
+    {yellow}--raw-content{reset} option to print the raw HTML content.
 
-    Only sort and order statements are supported for exporting, to filter results use the {yellow}database search{reset}
-    command.
-
-    \b
-    The {yellow}OUTPUT{reset} can be set to four different types:
-    {outputs}
+    {italic}Note{reset}: full color support is only enabled for truecolor terminals. If the terminal does not support
+    truecolor, the closest ANSI color match will be used instead.
     """
     db: Database = database()
 
-    db_table: Table = get_table(db, table)
-    column = column or db_table.columns
-
-    results: Cursor = db_table.select(None, column, [" ".join(s) for s in (sort or ((db_table.key.name, ""),))])
-    results_total: int = 0
-
-    if output == Output.csv:
-        results_total = print_csv(results, file, ",")
-    elif output == Output.tsv:
-        results_total = print_csv(results, file, "\t")
-    elif output == Output.json:
-        results_total = print_json(results, file)
-
-    if total:
-        echo(f"{blue}Total{reset}: {yellow}{results_total}{reset}", color=ctx.color)
+    if not (entry := get_table(db, table)[id_[0]]):
+        secho(f"Entry {id_[0]!r} could not be found in {table.lower()}", fg="red", color=ctx.color)
+    elif table == submissions_table:
+        echo(view_entry(entry, [SubmissionsColumns.DESCRIPTION.name], raw_html=raw_content), color=ctx.color)
+    elif table == journals_table:
+        echo(view_entry(entry, [JournalsColumns.CONTENT.name], raw_html=raw_content), color=ctx.color)
+    elif table == users_table:
+        echo(view_entry(entry, [UsersColumns.USERPAGE.name], raw_html=raw_content), color=ctx.color)
+    else:
+        echo(view_entry(entry, [], raw_html=raw_content), color=ctx.color)
 
 
 @database_app.command("remove", no_args_is_help=True, short_help="Remove entries.")
@@ -574,10 +696,10 @@ def database_add(ctx: Context, database: Callable[..., Database], table: str, fi
         if any(c.name.upper() not in data for c in db.submissions.columns):
             raise BadParameter(f"Missing fields {set(map(str.upper, db.submissions.columns)) - set(data.keys())}"
                                f" for table {table}",
-                               ctx, next((p for p in ctx.command.params if p.name == "file")))
+                               ctx, get_param(ctx, "file"))
         elif not replace and (id_ := data[idc := db.submissions.key.name.upper()]) in db.submissions:
             raise BadParameter(f"Entry with {idc} {id_!r} already exists in {table} table, but '--replace' is not set.",
-                               ctx, next((p for p in ctx.command.params if p.name == "file")))
+                               ctx, get_param(ctx, "file"))
         sub_file_orig, sub_thumb_orig = db.submissions.get_submission_files(data["id"])
         sub_file: bytes | None = None
         sub_thumb: bytes | None = None
@@ -600,10 +722,10 @@ def database_add(ctx: Context, database: Callable[..., Database], table: str, fi
         if any(c.name.upper() not in data for c in db_table.columns):
             raise BadParameter(f"Missing fields {set(map(str.upper, db_table.columns)) - set(data.keys())}"
                                f" for table {table}",
-                               ctx, next((p for p in ctx.command.params if p.name == "file")))
+                               ctx, get_param(ctx, "file"))
         elif not replace and (id_ := data[idc := db_table.key.name.upper()]) in db_table:
             raise BadParameter(f"Entry with {idc} {id_!r} already exists in {table} table, but '--replace' is not set.",
-                               ctx, next((p for p in ctx.command.params if p.name == "file")))
+                               ctx, get_param(ctx, "file"))
         try:
             db_table.insert(db_table.format_entry(data), replace=replace)
         finally:
@@ -638,19 +760,16 @@ def database_edit(ctx: Context, database: Callable[..., Database], table: str, _
     file.close()
 
     if not data and table.lower() != db.submissions.name.lower():
-        raise BadParameter(f"FILE cannot be empty for {table} table.", ctx,
-                           next(p for p in ctx.command.params if p.name == "file"))
+        raise BadParameter(f"FILE cannot be empty for {table} table.", ctx, get_param(ctx, "file"))
     elif not isinstance(data, dict):
-        raise BadParameter(f"Data must be in JSON object format.", ctx,
-                           next(p for p in ctx.command.params if p.name == "file"))
+        raise BadParameter(f"Data must be in JSON object format.", ctx, get_param(ctx, "file"))
 
     add_history(db, ctx, table=table, id=_id, file=file.name,
                 submission_file=submission_file.name if submission_file else None,
                 submission_thumbnail=submission_thumbnail.name if submission_thumbnail else None)
 
     if (entry := db_table[_id]) is None:
-        raise BadParameter(f"No entry with ID {_id} in {table}.", ctx,
-                           next(p for p in ctx.command.params if p.name == "_id"))
+        raise BadParameter(f"No entry with ID {_id} in {table}.", ctx, get_param(ctx, "_id"))
 
     if submission_file:
         ext: str = db.submissions.save_submission_file(_id, submission_file.read(), "submission", "")
@@ -789,7 +908,7 @@ database_app.list_commands = lambda *_: [
     database_info.name,
     database_history.name,
     database_search.name,
-    database_export.name,
+    database_view.name,
     database_add.name,
     database_remove.name,
     database_merge.name,
