@@ -11,11 +11,13 @@ from click import echo
 from click import group
 from click import option
 from click import pass_context
+from click import secho
 from click.shell_completion import CompletionItem
-from faapi import User
+from faapi import FAAPI
 from faapi.exceptions import Unauthorized
 from falocalrepo_database import Database
 from falocalrepo_database.database import clean_username
+from requests.exceptions import RequestException
 
 from .colors import *
 from .util import CompleteChoice
@@ -28,6 +30,7 @@ from .util import help_option
 from .util import open_api
 from ..downloader import Downloader
 from ..downloader import Folder
+from ..downloader import sort_set
 
 
 class FolderChoice(CompleteChoice):
@@ -40,6 +43,24 @@ class FolderChoice(CompleteChoice):
     ]
 
 
+class UpdateFolderChoice(CompleteChoice):
+    completion_items: list[CompletionItem] = [
+        *FolderChoice.completion_items,
+        CompletionItem(f"{Folder.watchlist_by.value}", help=f"User's watches"),
+        CompletionItem(f"{Folder.watchlist_to.value}", help=f"Users watching user"),
+    ]
+
+
+class DownloadFolderChoice(CompleteChoice):
+    completion_items: list[CompletionItem] = [
+        *FolderChoice.completion_items,
+        *[CompletionItem(f"{Folder.watchlist_by.value}:{f}", help=f"User's watches ({f})")
+          for f in Folder if f != Folder.watchlist_by and f != Folder.watchlist_to],
+        *[CompletionItem(f"{Folder.watchlist_to.value}:{f}", help=f"Users watching user ({f})")
+          for f in Folder if f != Folder.watchlist_by and f != Folder.watchlist_to],
+    ]
+
+
 output_option = option("--simple-output", is_flag=True, default=False, help="Simplified output.")
 dry_run_option = option("--dry-run", is_flag=True, default=False, help="Fetch entries without modifying database.")
 verbose_report_option = option("--verbose-report", is_flag=True, default=False, help="Output full report with IDs.")
@@ -59,9 +80,10 @@ def download_app():
 
 
 def users_callback(ctx: Context, param: Option, value: tuple[str]) -> tuple[str]:
-    value_clean: list[str] = list(filter(bool, map(clean_username, value)))
-    if not value_clean and param.required:
-        raise BadParameter("Invalid users", ctx, param)
+    if not value or ctx.params.get("like"):
+        return value
+    if not (value_clean := [u.lower() if u.lower() == "@me" else clean_username(u) for u in value]):
+        raise BadParameter("No valid users (allowed characters are [a-z0-9.~-]).", ctx, param)
     return tuple(sorted(set(value_clean), key=value_clean.index))
 
 
@@ -76,14 +98,14 @@ def download_login(ctx: Context, database: Callable[..., Database]):
     """
 
     db: Database = database()
+    api: FAAPI = open_api(db)
 
     echo(f"{bold}Login{reset}", color=ctx.color)
 
     try:
         echo(f"{blue}User{reset}: ", nl=False, color=ctx.color)
-        me: User = open_api(db).me()
-        echo(f"{green}{me.name}{reset}", color=ctx.color)
-    except Unauthorized as err:
+        echo(f"{green}{api.me().name}{reset}", color=ctx.color)
+    except (Unauthorized, RequestException) as err:
         echo(f"{red}{' '.join(err.args)}{reset}", color=ctx.color)
         ctx.exit(1)
 
@@ -91,8 +113,8 @@ def download_login(ctx: Context, database: Callable[..., Database]):
 @download_app.command("users", short_help="Download users.", no_args_is_help=True)
 @option("--user", "-u", "users", metavar="USER", required=True, multiple=True, type=str, callback=users_callback,
         help="Username.")
-@option("--folder", "-f", "folders", metavar="FOLDER", required=True, multiple=True, type=FolderChoice(),
-        callback=lambda _c, _p, v: sorted(set(v), key=v.index), help="Folder to download.")
+@option("--folder", "-f", "folders", metavar="FOLDER", required=True, multiple=True, type=DownloadFolderChoice(),
+        callback=lambda _c, _p, v: sort_set(v), help="Folder to download.")
 @dry_run_option
 @verbose_report_option
 @report_file_option
@@ -100,21 +122,39 @@ def download_login(ctx: Context, database: Callable[..., Database]):
 @color_option
 @help_option
 @pass_context
-@docstring_format(', '.join(Folder))
+@docstring_format(', '.join([c.value for c in FolderChoice.completion_items] +
+                            [Folder.watchlist_by.value + f":{yellow}FOLDER{reset}"] +
+                            [Folder.watchlist_to.value + f":{yellow}FOLDER{reset}"]))
 def download_users(ctx: Context, database: Callable[..., Database], users: tuple[str], folders: tuple[str],
                    dry_run: bool, verbose_report: bool, report_file: TextIO | None):
     """
     Download specific user folders, where {yellow}FOLDER{reset} is one of {0}. Multiple {yellow}--user{reset} and
-    {yellow}--folder{reset} arguments can be passed.
+    {yellow}--folder{reset} arguments can be passed. {yellow}USER{reset} can be set to {cyan}@me{reset} to fetch own
+    username. {cyan}watchlist-by:{yellow}FOLDER{reset} and {cyan}watchlist-to:{yellow}FOLDER{reset} arguments add the
+    specified {yellow}FOLDER{reset}(s) to the new user entries.
 
     The optional {yellow}--dry-run{reset} option disables downloading and saving and simply lists fetched entries.
     Users are not added/deactivated.
     """
     db: Database = database()
-    add_history(db, ctx, users=users, folders=folders)
+    if not dry_run:
+        add_history(db, ctx, users=users, folders=folders)
     downloader: Downloader = Downloader(db, open_api(db), color=ctx.color, dry_run=dry_run)
+    watchlist_by: list[str] = [f.split(":")[1] for f in folders if f.startswith(Folder.watchlist_by.value)]
+    watchlist_to: list[str] = [f.split(":")[1] for f in folders if f.startswith(Folder.watchlist_to.value)]
+    folders_ = [f for f in folders
+                if not (f.startswith(Folder.watchlist_by.value) or f.startswith(Folder.watchlist_to.value))]
+    if watchlist_by:
+        folders_.insert(folders.index(next((f for f in folders if f.startswith(Folder.watchlist_by.value)))),
+                        f"{Folder.watchlist_by.value}:{':'.join(watchlist_by)}")
+    if watchlist_to:
+        folders_.insert(folders.index(next((f for f in folders if f.startswith(Folder.watchlist_to.value)))),
+                        f"{Folder.watchlist_to.value}:{':'.join(watchlist_to)}")
+    folders = folders_
     try:
         downloader.download_users(list(users), list(folders))
+    except RequestException as err:
+        secho(f"\nError: An error occurred during download: {err!r}.", fg="red", color=ctx.color)
     finally:
         echo()
         downloader.verbose_report() if verbose_report else downloader.report()
@@ -125,11 +165,12 @@ def download_users(ctx: Context, database: Callable[..., Database], users: tuple
 @download_app.command("update", short_help="Download new entries for users in database.")
 @option("--user", "-u", "users", metavar="USER", multiple=True, type=str, callback=users_callback,
         help="User to update.")
-@option("--folder", "-f", "folders", metavar="FOLDER", multiple=True, type=FolderChoice(),
-        callback=lambda _c, _p, v: sorted(set(v), key=v.index), help="Folder to update.")
+@option("--folder", "-f", "folders", metavar="FOLDER", multiple=True, type=UpdateFolderChoice(),
+        callback=lambda _c, _p, v: sort_set(v), help="Folder to update.")
 @option("--stop", metavar="STOP", type=IntRange(0, min_open=True), default=1, show_default=True,
         help="Number of submissions to find in the database before stopping.")
 @option("--deactivated", is_flag=True, default=False, help="Check deactivated users.")
+@option("--like", is_flag=True, is_eager=True, default=False, help=f"Consider {yellow}USER{reset} to be LIKE queries.")
 @dry_run_option
 @verbose_report_option
 @report_file_option
@@ -137,14 +178,14 @@ def download_users(ctx: Context, database: Callable[..., Database], users: tuple
 @color_option
 @help_option
 @pass_context
-@docstring_format(', '.join(Folder))
+@docstring_format(', '.join(c.value for c in UpdateFolderChoice.completion_items))
 def download_update(ctx: Context, database: Callable[..., Database], users: tuple[str], folders: tuple[str], stop: int,
-                    deactivated: bool, dry_run: bool, verbose_report: bool, report_file: TextIO | None):
+                    deactivated: bool, like: bool, dry_run: bool, verbose_report: bool, report_file: TextIO | None):
     """
     Download new entries using the users and folders already in the database. {yellow}--user{reset} and
     {yellow}--folder{reset} options can be used to restrict the update to specific users and or folders, where
     {yellow}FOLDER{reset} is one of {0}. Multiple {yellow}--user{reset} and {yellow}--folder{reset} arguments can be
-    passed.
+    passed. {yellow}USER{reset} can be set to {cyan}@me{reset} to fetch own username.
 
     If the {yellow}--deactivated{reset} option is used, deactivated users are fetched instead of ignore. If the user is
     no longer inactive, the database entry will be modified as well.
@@ -152,14 +193,20 @@ def download_update(ctx: Context, database: Callable[..., Database], users: tupl
     The {yellow}--stop{reset} option allows setting after how many entries of each folder should be found in the
     database before stopping the update.
 
+    The {yellow}--like{reset} option enables using SQLite LIKE statements for {yellow}USER{reset} values, allowing to
+    select multiple users at once.
+
     The optional {yellow}--dry-run{reset} option disables downloading and saving and simply lists fetched entries.
     Users are not added/deactivated.
     """
     db: Database = database()
-    add_history(db, ctx, users=users, folders=folders, stop=stop)
+    if not dry_run:
+        add_history(db, ctx, users=users, folders=folders, stop=stop)
     downloader: Downloader = Downloader(db, open_api(db), color=ctx.color, dry_run=dry_run)
     try:
-        downloader.download_users_update(list(users), list(folders), stop, deactivated)
+        downloader.download_users_update(list(users), list(folders), stop, deactivated, like)
+    except RequestException as err:
+        secho(f"\nError: An error occurred during download: {err!r}.", fg="red", color=ctx.color)
     finally:
         echo()
         downloader.verbose_report() if verbose_report else downloader.report()
@@ -190,10 +237,13 @@ def download_submissions(ctx: Context, database: Callable[..., Database], submis
     The optional {yellow}--dry-run{reset} option disables downloading and saving and simply lists fetched entries
     """
     db: Database = database()
-    add_history(db, ctx, submission_id=submission_id, replace=replace)
+    if not dry_run:
+        add_history(db, ctx, submission_id=submission_id, replace=replace)
     downloader: Downloader = Downloader(db, open_api(db), color=ctx.color, dry_run=dry_run)
     try:
         downloader.download_submissions(list(submission_id), replace)
+    except RequestException as err:
+        secho(f"\nError: An error occurred during download: {err!r}.", fg="red", color=ctx.color)
     finally:
         echo()
         downloader.verbose_report() if verbose_report else downloader.report()
@@ -224,10 +274,13 @@ def download_journals(ctx: Context, database: Callable[..., Database], journal_i
     The optional {yellow}--dry-run{reset} option disables downloading and saving and simply lists fetched entries.
     """
     db: Database = database()
-    add_history(db, ctx, journal_id=journal_id, replace=replace)
+    if not dry_run:
+        add_history(db, ctx, journal_id=journal_id, replace=replace)
     downloader: Downloader = Downloader(db, open_api(db), color=ctx.color, dry_run=dry_run)
     try:
         downloader.download_journals(list(journal_id), replace)
+    except RequestException as err:
+        secho(f"\nError: An error occurred during download: {err!r}.", fg="red", color=ctx.color)
     finally:
         echo()
         downloader.verbose_report() if verbose_report else downloader.report()
