@@ -13,12 +13,14 @@ from typing import TypeVar
 from click import echo
 from faapi import FAAPI
 from faapi import SubmissionPartial
+from faapi import UserPartial
 from faapi.exceptions import DisabledAccount
 from faapi.exceptions import NotFound
 from faapi.exceptions import NoticeMessage
 from faapi.exceptions import ServerError
 from falocalrepo_database import Column
 from falocalrepo_database import Database
+from falocalrepo_database.selector import SelectorBuilder as Sb
 from falocalrepo_database.tables import JournalsColumns
 from falocalrepo_database.tables import SubmissionsColumns
 from falocalrepo_database.tables import UsersColumns
@@ -31,6 +33,7 @@ from .console.util import clean_string
 _FolderDownloader = Callable[[str, int | str], tuple[list[SubmissionPartial], str | int]]
 
 T = TypeVar("T")
+P = TypeVar("P")
 
 
 class OutputType(str, Enum):
@@ -44,6 +47,8 @@ class Folder(str, Enum):
     favorites = "favorites"
     journals = "journals"
     userpage = "userpage"
+    watchlist_by = "watchlist-by"
+    watchlist_to = "watchlist-to"
 
     @classmethod
     def as_list(cls: EnumMeta) -> list[str]:
@@ -138,6 +143,7 @@ class Bar:
         self.message_: str = message
 
 
+# noinspection DuplicatedCode
 class Downloader:
     def __init__(self, db: Database, api: FAAPI, *, color: bool = True, dry_run: bool = False):
         self.db: Database = db
@@ -161,7 +167,6 @@ class Downloader:
         self.thumbnail_errors: list[int] = []
         self.journal_errors: list[int] = []
 
-    # noinspection DuplicatedCode
     def report(self):
         items: list[tuple[str, int]] = [
             ("Added users", len(set(self.added_users))),
@@ -236,32 +241,32 @@ class Downloader:
         return self._bar
 
     def bar_update(self, total: int, level: int):
-        if self.output == OutputType.simple:
+        if self._bar is None or self.output == OutputType.simple:
             return
-        self.bar().update(total, level)
+        self._bar.update(total, level)
 
     def bar_message(self, message: str, color: str = "", *, always: bool = False):
-        if self.output == OutputType.simple:
+        if self._bar is None or self.output == OutputType.simple:
             if always:
                 echo(message)
             return
-        self.bar().message(message, color)
+        self._bar.message(message, color)
 
     def bar_close(self, end: str = "\n"):
         if self._bar is None or self.output == OutputType.simple:
             return
-        self.bar().close(end)
+        self._bar.close(end)
         self._bar = None
 
     def bar_clear(self):
         if self._bar is None or self.output == OutputType.simple:
             return
-        self.bar().clear()
+        self._bar.clear()
 
     def bar_delete(self):
         if self._bar is None or self.output == OutputType.simple:
             return
-        self.bar().delete()
+        self._bar.delete()
 
     def download_bytes(self, url: str) -> bytes | None:
         try:
@@ -318,140 +323,177 @@ class Downloader:
         self.thumbnail_errors += [] if thumb else [submission_id]
         return 0
 
-    # noinspection DuplicatedCode
-    def download_user_journals(self, user: str, stop: int = -1, clear_last_found: bool = False) -> int:
-        page: int = 1
+    def download_user_folder(self, user: str, folder: Folder, downloader_entries: Callable[[str, P], tuple[list[T], P]],
+                             page_start: P, entry_id_getter: Callable[[T], int | str], entry_formats: tuple[str, str],
+                             contains: Callable[[T], dict | None],
+                             modify_checks: list[tuple[Callable[[T, dict], bool], str]],
+                             save: tuple[Callable[[T], int | None], str], stop: int = -1,
+                             clear_last_found: bool = False, clear_found: bool = False,
+                             ) -> tuple[int, tuple[list[int | str], list[int | str], list[int | str]]]:
+        entries_added: list[int | str] = []
+        entries_modified: list[int | str] = []
+        entries_errors: list[int | str] = []
+        page: P | None = page_start
+        page_i: int = 0
         while page:
-            page_width: int = len(str(page))
-            page_id_width: int = page_width + 4 + 10
-            folder_page_width: int = len(user) + 1 + len(Folder.journals.name) + 1 + page_width
-            padding: int = (w - folder_page_width - self.bar_width - 2 - 1) if (w := terminal_width()) else 0
-            echo(f"{yellow}{user}{reset}/{yellow}{Folder.journals.name}{reset} {page}" + (" " * padding),
-                 nl=self.output == OutputType.simple, color=self.color)
-            self.bar()
-            self.bar_message("DOWNLOAD")
-            result, err = download_catch(self.api.journals, user, page)
-            if err:
-                self.user_errors += [user]
-                self.err_to_bar(err)
-                return err
-            self.bar_close("")
-            self.clear_line()
-            journals, next_page = result
-            for i, journal in enumerate(journals, 1):
-                title_width: int = w - page_id_width - 1 - self.bar_width - 2 - 1 - 1 if (w := terminal_width()) else 0
-                echo(("\r" * (self.output == OutputType.rich)) + f"{page}/{i:02} {blue}{journal.id:010}{reset} " +
-                     fit_string(clean_string(journal.title), title_width).ljust(title_width) + " ",
-                     nl=self.output == OutputType.simple, color=self.color)
-                self.bar()
-                self.bar_message("SEARCHING")
-                if journal.id in self.db.journals:
-                    self.bar_message("IN DB", green, always=True)
-                    if self.dry_run:
-                        stop -= 1
-                        if clear_last_found:
-                            self.bar_close("")
-                            self.clear_line()
-                        else:
-                            self.bar_close()
-                    elif self.db.journals.set_user_update(journal.id, 1):
-                        self.db.commit()
-                        self.bar_message("UPDATED", green, always=True)
-                        self.modified_journals += [journal.id]
-                    else:
-                        stop -= 1
-                        if clear_last_found:
-                            self.bar_close("")
-                            self.clear_line()
-                        else:
-                            self.bar_close()
-                elif self.dry_run:
-                    self.bar_message("SKIPPED", green)
-                    self.bar_close()
-                else:
-                    self.db.journals.save_journal({
-                        **format_entry(dict(journal) | {"author": journal.author.name}, self.db.journals.columns),
-                        JournalsColumns.USERUPDATE.value.name: 1,
-                    })
-                    self.db.commit()
-                    self.bar_message("ADDED", green)
-                    self.bar_close()
-                    self.added_journals += [journal.id]
-                if stop == 0:
-                    return 0
-            page = next_page
-
-    # noinspection DuplicatedCode
-    def download_user_submissions_folder(self, user: str, folder: Folder, stop: int = -1,
-                                         clear_last_found: bool = False) -> int:
-        page: int = 0
-        next_page: int | str = "/" if folder == Folder.favorites else 1
-        downloader: _FolderDownloader = get_downloader(self.api, folder)
-        while next_page:
-            page += 1
-            page_width: int = len(str(page))
-            page_id_width: int = page_width + 4 + 10
+            page_i += 1
+            page_width: int = len(str(page_i))
             folder_page_width: int = len(user) + 1 + len(folder.name) + 1 + page_width
             padding: int = (w - folder_page_width - self.bar_width - 2 - 1) if (w := terminal_width()) else 0
-            echo(f"{yellow}{user}{reset}/{yellow}{folder.name}{reset} {page}" + (" " * padding),
+            echo(f"{yellow}{user}{reset}/{yellow}{folder.name}{reset} {page_i}" +
+                 (" " * padding),
                  nl=self.output == OutputType.simple, color=self.color)
             self.bar()
             self.bar_message("DOWNLOAD")
-            result, err = download_catch(downloader, user, next_page)
+            result, err = download_catch(downloader_entries, user, page)
             if err:
                 self.user_errors += [user]
                 self.err_to_bar(err)
-                return err
+                return err, (entries_added, entries_modified, entries_errors)
             self.bar_close("")
             self.clear_line()
-            submissions, next_page = result
-            for i, sub_partial in enumerate(submissions, 1):
-                title_width: int = w - page_id_width - 1 - self.bar_width - 2 - 1 - 1 if (w := terminal_width()) else 0
-                echo(('\r' * (self.output == OutputType.rich)) + f"{page}/{i:02} {blue}{sub_partial.id:010}{reset} " +
-                     fit_string(clean_string(sub_partial.title), title_width).ljust(title_width) + " ",
+            entries: list[T] = result[0]
+            page = result[1]
+            entries_width: int = len(str(len(entries)))
+            for i, entry in enumerate(entries, 1):
+                t_width: int = terminal_width()
+                available_space: int = t_width - self.bar_width - 2 - 1 - 1
+                entry_num: str = f"{page_i}/{i:0{entries_width}}"
+                entry_id: str = clean_string(entry_formats[0].format(entry).strip())
+                entry_title: str = clean_string(entry_formats[1].format(entry).strip())
+                entry_outputs: list[str] = list(filter(bool, [entry_num, entry_id, entry_title]))
+                padding = available_space - len(" ".join(entry_outputs))
+                while entry_outputs and t_width and (padding := available_space - len(" ".join(entry_outputs))) < 0:
+                    if (space_last := available_space - len(" ".join(entry_outputs[:-1]))) < 1:
+                        entry_outputs.pop()
+                    else:
+                        entry_outputs[-1] = fit_string(entry_outputs[-1], space_last - 1)
+                padding = 0 if padding < 0 else padding
+                entry_outputs = [entry_outputs[0] if entry_outputs else "",
+                                 (blue + entry_outputs[1] + reset) if entry_outputs[1:] else "",
+                                 entry_outputs[2] if entry_outputs[2:] else ""]
+                entry_output: str = " ".join(filter(bool, entry_outputs)) + " "
+                echo(("\r" if self.output == OutputType.rich else "") + entry_output + (" " * padding),
                      nl=self.output == OutputType.simple, color=self.color)
                 self.bar()
                 self.bar_message("SEARCHING")
-                if sub_partial.id in self.db.submissions:
+                if curr_entry := contains(entry):
                     self.bar_message("IN DB", green, always=True)
                     if self.dry_run:
                         stop -= 1
-                        if clear_last_found and stop == 0:
+                        if clear_found or (clear_last_found and stop == 0):
                             self.bar_close("")
                             self.clear_line()
-                        else:
-                            self.bar_close()
-                    elif folder != Folder.favorites and (self.db.submissions.set_user_update(sub_partial.id, 1) or
-                                                         self.db.submissions.set_folder(sub_partial.id, folder.value)):
-                        self.db.commit()
-                        self.bar_message("UPDATED", green, always=True)
-                        self.modified_submissions += [sub_partial.id]
-                        self.bar_close()
-                    elif folder == Folder.favorites and self.db.submissions.add_favorite(sub_partial.id, user):
-                        self.db.commit()
-                        self.bar_message("ADDED FAV", green, always=True)
-                        self.modified_submissions += [sub_partial.id]
-                        self.bar_close()
                     else:
-                        stop -= 1
-                        if clear_last_found and stop == 0:
-                            self.bar_close("")
-                            self.clear_line()
-                        else:
-                            self.bar_close()
+                        modified: bool = False
+                        for check, message in modify_checks:
+                            if modified := check(entry, curr_entry):
+                                self.db.commit()
+                                self.bar_message(message or "UPDATED", green, always=True)
+                                entries_modified.append(entry_id_getter(entry))
+                                break
+                        if not modified:
+                            stop -= 1
+                            if clear_found or (clear_last_found and stop == 0):
+                                self.bar_close("")
+                                self.clear_line()
                 elif self.dry_run:
                     self.bar_message("SKIPPED", green)
-                    self.bar_close()
                 else:
-                    self.download_submission(sub_partial.id, int(folder != Folder.favorites),
-                                             [user] if folder == Folder.favorites else None, sub_partial.thumbnail_url)
+                    err = save[0](entry)
+                    if not self.err_to_bar(err) and save[1]:
+                        self.bar_message(save[1], green, always=True)
+                    entries_added.append(entry_id_getter(entry))
+                self.bar_close()
                 if stop == 0:
-                    return 0
+                    page = None
+                    break
             self.clear_line()
+        return 0, (entries_added, entries_modified, entries_errors)
+
+    def download_user_journals(self, user: str, stop: int = -1, clear_last_found: bool = False) -> int:
+        err, [entries_added, entries_modified, entries_errors] = self.download_user_folder(
+            user=user, folder=Folder.journals, downloader_entries=self.api.journals, page_start=1,
+            entry_id_getter=lambda j: j.id,
+            entry_formats=("{0.id:010}", "{0.title}"),
+            contains=lambda j: self.db.journals[j.id],
+            modify_checks=[(lambda journal, _: self.db.journals.set_user_update(journal.id, 1), "")],
+            save=(lambda journal: self.db.journals.save_journal(
+                {**format_entry(dict(journal) | {"author": journal.author.name}, self.db.journals.columns),
+                 JournalsColumns.USERUPDATE.value.name: 1, }), "ADDED"),
+            stop=stop, clear_last_found=clear_last_found
+        )
+        self.added_journals.extend(entries_added)
+        self.modified_journals.extend(entries_modified)
+        self.journal_errors.extend(entries_errors)
+        return err
+
+    def download_user_submissions(self, user: str, folder: Folder, stop: int = -1,
+                                  clear_last_found: bool = False) -> int:
+        downloader: _FolderDownloader = get_downloader(self.api, folder)
+        page_start: int | str = "/" if folder == Folder.favorites else 1
+        modify_checks: list[tuple[Callable[[SubmissionPartial, dict], bool], str]]
+
+        if folder == Folder.favorites:
+            modify_checks = [(lambda submission, _: self.db.submissions.add_favorite(submission.id, user),
+                              "ADDED FAV")]
+        else:
+            modify_checks = [(lambda submission, _: (self.db.submissions.set_user_update(submission.id, 1) +
+                                                     self.db.submissions.set_folder(submission.id, folder.value)),
+                              "UPDATED")]
+
+        err, [_entries_added, entries_modified, _entries_errors] = self.download_user_folder(
+            user=user, folder=folder, downloader_entries=downloader, page_start=page_start,
+            entry_id_getter=lambda s: s.id,
+            entry_formats=("{0.id:010}", "{0.title}"),
+            contains=lambda s: self.db.submissions[s.id],
+            modify_checks=modify_checks,
+            save=(lambda sub_partial: self.download_submission(
+                sub_partial.id, int(folder != Folder.favorites),
+                [user] if folder == Folder.favorites else None,
+                sub_partial.thumbnail_url), ""),
+            stop=stop, clear_last_found=clear_last_found
+        )
+        self.modified_submissions.extend(entries_modified)
+        return err
+
+    def download_user_watchlist(self, user: str, watchlist: Folder, folders: list[str], clear_found: bool = False
+                                ) -> int:
+        downloader: Callable[[str, int], tuple[list[UserPartial], int]]
+
+        if watchlist == Folder.watchlist_by:
+            downloader = self.api.watchlist_by
+        else:
+            downloader = self.api.watchlist_to
+
+        def check_folders(db: Database, watch: UserPartial, entry: dict[str, Any]) -> bool:
+            if folders_ := [f for f in folders if f not in [f_.strip("!") for f_ in entry[UsersColumns.FOLDERS.name]]]:
+                active: str = "!" if any(f.startswith("!") for f in entry[UsersColumns.FOLDERS.name]) else ""
+                for folder in folders_:
+                    db.users.add_folder(watch.name_url, active + folder)
+                return True
+            return False
+
+        err, [entries_added, entries_modified, _entries_errors] = self.download_user_folder(
+            user=user, folder=watchlist, downloader_entries=downloader, page_start=1,
+            entry_id_getter=lambda u: u.name_url,
+            entry_formats=("{0.status}{0.name}", ""),
+            contains=lambda u: self.db.users[u.name_url],
+            modify_checks=[(lambda w, e: check_folders(self.db, w, e), "UPDATED")],
+            save=(lambda watch: self.db.users.save_user(
+                {UsersColumns.USERNAME.value.name: watch.name_url,
+                 UsersColumns.FOLDERS.value.name: set(folders),
+                 UsersColumns.USERPAGE.value.name: ""}), "ADDED"),
+            stop=-1, clear_found=clear_found
+        )
+        self.added_users.extend(entries_added)
+        self.modified_users.extend(entries_modified)
+        return 1
 
     def download_user_page(self, username: str, clear_found: bool = False) -> int:
         padding: int = w - self.bar_width - 2 - 1 if (w := terminal_width()) else 0
-        echo(f"{yellow}{username[:padding or None]:<{padding}}{reset}", nl=self.output == OutputType.simple)
+        echo(f"{yellow}{username[:padding or None]:<{padding}}{reset}", nl=self.output == OutputType.simple,
+             color=self.color)
         self.bar()
         if self.dry_run:
             self.bar_message("SKIPPED", green)
@@ -477,11 +519,31 @@ class Downloader:
         self.bar_message("ADDED" if added else "UPDATED", green, always=True)
         return 0
 
+    def download_me(self) -> tuple[str, int]:
+        padding: int = w - self.bar_width - 2 - 1 if (w := terminal_width()) else 0
+        echo(f"Downloading: {yellow}@me{reset}/{yellow}username{reset}", color=self.color)
+        echo(f"{blue}{'@me':<{padding}}{reset}", nl=self.output == OutputType.simple,
+             color=self.color)
+        self.bar()
+        self.bar_message("DOWNLOAD")
+        user, err = download_catch(self.api.me)
+        if self.err_to_bar(err) or not user:
+            self.user_errors += ["@me"]
+            return "", err
+        echo(("\r" if self.output == OutputType.rich else "") +
+             f"{blue}@me{reset} {fit_string(repr(user), padding - 4).ljust(padding - 4)}",
+             nl=self.output == OutputType.simple, color=self.color)
+        self.bar_close("")
+        self.bar()
+        self.bar_message("FOUND", green, always=True)
+        self.bar_close()
+        return user.name_url, err
+
     def _download_users(self, users_folders: Iterable[tuple[str, list[str]]], stop: int = -1):
         operation: str = "Downloading" if stop < 0 else "Updating"
         for user, folders in users_folders:
             for folder in folders:
-                echo(f"{operation}: {yellow}{user}{reset}/{yellow}{folder}{reset}", color=self.color)
+                echo(f"{operation}: {yellow}{user}{reset}/{yellow}{folder.split(':')[0]}{reset}", color=self.color)
                 user_added: bool = False
                 if not self.dry_run:
                     if user_added := user not in self.db.users:
@@ -491,14 +553,30 @@ class Downloader:
                         self.db.commit()
                         self.added_users += [user]
                     self.db.users.activate(user)
+                    if folder.startswith(w := Folder.watchlist_by) and \
+                            (wfs := [f for f in self.db.users[user][UsersColumns.FOLDERS.name] if f.startswith(w)]):
+                        for wf in filter(lambda f: f != folder, wfs):
+                            self.db.users.remove_folder(user, wf)
+                            self.modified_users += [user]
+                    elif folder.startswith(w := Folder.watchlist_to) and \
+                            (wfs := [f for f in self.db.users[user][UsersColumns.FOLDERS.name] if f.startswith(w)]):
+                        for wf in filter(lambda f: f != folder, wfs):
+                            self.db.users.remove_folder(user, wf)
+                        self.modified_users += [user]
                     self.db.users.add_folder(user, folder)
                 err: int
                 if folder == Folder.userpage:
                     err = self.download_user_page(user, stop == 1)
                 elif folder == Folder.journals:
                     err = self.download_user_journals(user, stop, stop == 1)
+                elif folder in (Folder.gallery, Folder.scraps, Folder.favorites):
+                    err = self.download_user_submissions(user, Folder[folder.lower()], stop, stop == 1)
+                elif folder.startswith(Folder.watchlist_by.value):
+                    err = self.download_user_watchlist(user, Folder.watchlist_by, folder.split(":")[1:], stop == 1)
+                elif folder.startswith(Folder.watchlist_to.value):
+                    err = self.download_user_watchlist(user, Folder.watchlist_to, folder.split(":")[1:], stop == 1)
                 else:
-                    err = self.download_user_submissions_folder(user, Folder[folder.lower()], stop, stop == 1)
+                    raise Exception(f"Unknown folder {folder}")
                 if err in (1, 2):
                     if self.dry_run:
                         pass
@@ -514,24 +592,41 @@ class Downloader:
                 self.db.commit()
 
     def download_users(self, users: list[str], folders: list[str]):
+        if "@me" in users:
+            if me := self.download_me()[0]:
+                users[users.index("@me")] = me
+            else:
+                users.remove("@me")
+
         self._download_users([(u, folders) for u in users])
 
-    def download_users_update(self, users: list[str], folders: list[str], stop: int, deactivated: bool):
-        for user in [u for u in users if u not in self.db.users]:
-            padding: int = terminal_width() - 1 - self.bar_width - 2
-            echo(f"{green}{user:<{padding}}{reset}", nl=self.output == OutputType.simple, color=self.color)
-            self.bar()
-            self.bar_message("NOT IN DB", red, always=True)
-            self.bar_close()
+    def download_users_update(self, users: list[str], folders: list[str], stop: int, deactivated: bool, like: bool):
+        if not like:
+            for user in [u for u in users if u != "@me" and u not in self.db.users]:
+                padding: int = terminal_width() - 1 - self.bar_width - 2
+                echo(f"{green}{user:<{padding}}{reset}", nl=self.output == OutputType.simple, color=self.color)
+                self.bar()
+                self.bar_message("NOT IN DB", red, always=True)
+                self.bar_close()
 
-        users_cursor: Iterable[dict] = self.db.users[users] if users else self.db.users.select(
-            order=[UsersColumns.USERNAME.value.name])
+        if "@me" in users:
+            if me := self.download_me()[0]:
+                users[users.index("@me")] = me
+            else:
+                users.remove("@me")
+
+        users_cursor: Iterable[dict]
+        if like and users:
+            users_cursor = self.db.users.select(Sb() | [Sb(UsersColumns.USERNAME.value.name) % u for u in users])
+        elif users:
+            users_cursor = self.db.users[users]
+        else:
+            users_cursor = self.db.users.select(order=[UsersColumns.USERNAME.value.name])
         users_folders: Iterable[tuple[str, list[str]]]
-        users_folders = ((u[UsersColumns.USERNAME.value.name],
-                          [f for f in u[UsersColumns.FOLDERS.value.name]])
+        users_folders = ((u[UsersColumns.USERNAME.value.name], [f for f in u[UsersColumns.FOLDERS.value.name]])
                          for u in users_cursor)
 
-        users_folders = sorted(users_folders, key=lambda uf: users.index(uf[0]) if users else uf[0])
+        users_folders = sorted(users_folders, key=lambda uf: users.index(uf[0]) if users and not like else uf[0])
 
         if deactivated:
             users_folders = ((u, [*map(lambda f: f.strip("!"), fs)]) for u, fs in users_folders)
@@ -539,10 +634,12 @@ class Downloader:
             users_folders = ((u, fs) for u, fs in users_folders if not any("!" in f for f in fs))
 
         if folders:
-            users_folders = ((u, sorted(filter(folders.__contains__, fs), key=folders.index))
+            users_folders = ((u, sorted(filter(lambda f: f.split(":")[0] in folders, fs),
+                                        key=lambda f: folders.index(f.split(":")[0])))
                              for u, fs in users_folders)
         else:
-            users_folders = ((u, sorted(fs, key=Folder.as_list().index)) for u, fs in users_folders)
+            users_folders = ((u, sorted(fs, key=lambda f: Folder.as_list().index(f.split(":")[0])))
+                             for u, fs in users_folders)
 
         self._download_users(users_folders, stop)
 
